/*
 * PROGRAM:
 * vtkcellgen
 * 
 * AUTHOR:
 * Soeren Gebbert
 * soerengebbert <at> googlemail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * The GNU General Public License is available here:
 * http://www.gnu.org/licenses/gpl.txt
 * 
 */


#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include "globals.h"

static int print_scalars(FILE * out, parameters *params, int xnum, int ynum, int znum, const char *id);
static int print_vectors(FILE * out, parameters *params, int xnum, int ynum, int znum, const char *id);
static int print_tensors(FILE * out, parameters *params, int xnum, int ynum, int znum, const char *id);
   

/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
void
print_ugrid_point_data (FILE * out, parameters *params)
{
  int xnum = params->xnum;
  int ynum = params->ynum;
  int znum = params->znum;

  /*This is the beginning of the point data section in the VTK file*/
  fprintf (out, "POINT_DATA %i\n", xnum * ynum * znum);

  if (params->quite == 0)
    {
      printf ("Creating point data: scalars\n");
    }

  /*print node numbering*/
  params->flag_pdata = 1;
  print_scalars(out, params, xnum, ynum, znum, "node_numbers");

  /*one scalar field*/
  params->flag_pdata = 0;
  print_scalars(out, params, xnum, ynum, znum, "node_scalars");
   
  if (params->quite == 0)
    {
      printf ("Creating point data: vectors\n");
    }

  /*and one vector field*/
  print_vectors(out, params, xnum, ynum, znum, "node_vectors");
   
  if (params->quite == 0)
    {
      printf ("Creating point data: tensors\n");
    }

  /*and one vector field*/
  print_tensors(out, params, xnum, ynum, znum, "node_tensors");



  return;
}

/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
void
print_ugrid_cell_data (FILE * out, parameters *params)
{
  int xnum = params->cellx;
  int ynum = params->celly;
  int znum = 1;
  int cell_vol;

  cell_vol = get_number_of_cells_per_volume(params->celltype);


  /*This is the beginning of the cell data section in the VTK file*/
  fprintf (out, "CELL_DATA %i\n", xnum * ynum * znum * cell_vol);

  if (params->quite == 0)
    {
      printf ("Creating cell data: scalars\n");
    }

  /*print cell numbering*/
  params->flag_pdata = 1;
  print_scalars(out, params, xnum, ynum, znum * cell_vol, "cell_numbers");

  /*one scalar field*/
  params->flag_pdata = 0;
  print_scalars(out, params, xnum, ynum, znum * cell_vol, "cell_scalars");
   
  if (params->quite == 0)
    {
      printf ("Creating cell data: vectors\n");
    }

  /*and one vector field*/
  print_vectors(out, params, xnum, ynum, znum * cell_vol, "cell_vectors");
   
  if (params->quite == 0)
    {
      printf ("Creating cell data: tensors\n");
    }

  /*and one vector field*/
  print_tensors(out, params, xnum, ynum, znum * cell_vol, "cell_tensors");


  return;
}


/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
int print_scalars(FILE * out, parameters *params, int xnum, int ynum, int znum, const char *id)
{
  int i, j, k;
  double dx = params->dx;
  double dy = params->dy; 
  double dz = params->dz; 
  int pdata = params->flag_pdata;
  ef_pointer func;
   
  func = get_elevfunction(params);

  if (pdata == 1)
    func = elevfunc_numbering;

  fprintf (out, "SCALARS %s float 1\n", id);
  fprintf (out, "LOOKUP_TABLE default\n");

  for (k = 0; k < znum; k++)
    for (j = 0; j < ynum; j++)
      for (i = 0; i < xnum; i++)
	  fprintf (out, "%g ", func(xnum, ynum, znum, i, j, k, dx, dy, dz));

  fprintf (out, "\n");
 
   return 1;
}


/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
int print_vectors(FILE * out, parameters *params, int xnum, int ynum, int znum, const char *id)
{
  int i, j, k;
  double dx = params->dx;
  double dy = params->dy; 
  double dz = params->dz; 
  sf_pointer sfunc;
  double x[3];
  
  sfunc = get_shapefunction(params);

  fprintf (out, "VECTORS %s float\n", id);

  for (k = 0; k < znum; k++)
    for (j = 0; j < ynum; j++)
      for (i = 0; i < xnum; i++)
        {
	  sfunc(xnum, ynum, znum, i, j, k, dx, dy, dz, x);
	  fprintf (out, "%g %g %g\n", x[0], x[1], x[2]);
	}

  fprintf (out, "\n");
  
  return 1;
}


/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
int print_tensors(FILE * out, parameters *params, int xnum, int ynum, int znum, const char *id)
{
  int i, j, k;
  double dx = params->dx;
  double dy = params->dy; 
  double dz = params->dz; 
  sf_pointer sfunc;
  double x[3];
  
  sfunc = get_shapefunction(params);

  fprintf (out, "TENSORS %s float\n", id);

  for (k = 0; k < znum; k++)
    for (j = 0; j < ynum; j++)
      for (i = 0; i < xnum; i++)
        {
	  sfunc(xnum, ynum, znum, i, j, k, dx, dy, dz, x);
	  fprintf (out, "%g %g %g\n", x[0],  0.0,  0.0);
	  fprintf (out, "%g %g %g\n",  0.0, x[1],  0.0);
	  fprintf (out, "%g %g %g\n",  0.0,  0.0, x[2]);
	}

  fprintf (out, "\n");
  
  return 1;
}
