/*
 * PROGRAM:
 * vtkcellgen
 * 
 * PURPOSE:
 * vtkcellgen is a very simple cell generator to supports the generation
 * of several linear, quadratic and cubic VTK cell types.
 * Based on one cell type a flat plate (unstructured grid) will be created with a specific number of 
 * cells in x and y direction. The output is written as
 * VTK legacy ASCII format to stdout or a specified file.
 *
 * AUTHOR:
 * Soeren Gebbert
 * soerengebbert <at> googlemail.com
 *
 * DATE:
 * Berlin, 02 Jul 2007
 *
 * VERSION:
 * 0.3
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *
 * The GNU General Public License is available here:
 * http://www.gnu.org/licenses/gpl.txt
 * 
 */

#ifndef __GLOBALS_H__
#define __GLOBALS_H__

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/*default grid size*/
#define CELLNUMX 1
#define CELLNUMY 1
#define XLENGTH 1.0
#define YLENGTH 1.0
#define ZLENGTH 1.0
#define DEFAULT_CELL VTK_QUAD


/*available cell types*/
#define  VTK_POLY_VERTEX		       2
#define  VTK_TRIANGLE                          5
#define  VTK_QUAD                              9
#define  VTK_TETRA                             10
#define  VTK_HEXAHEDRON			       12
#define  VTK_WEDGE                             13
#define  VTK_QUADRATIC_TRIANGLE                22
#define  VTK_QUADRATIC_QUAD                    23
#define  VTK_QUADRATIC_HEXAHEDRON              25
#define  VTK_QUADRATIC_WEDGE                   26
#define  VTK_BIQUADRATIC_QUAD                  28
#define  VTK_TRIQUADRATIC_HEXAHEDRON           29
#define  VTK_QUADRATIC_LINEAR_QUAD             30
#define  VTK_QUADRATIC_LINEAR_WEDGE            31
#define  VTK_BIQUADRATIC_QUADRATIC_WEDGE       32
#define  VTK_BIQUADRATIC_QUADRATIC_HEXAHEDRON  33
#define  VTK_CUBIC_EDGE                        34
#define  VTK_BICUBIC_QUAD                      36
#define  VTK_TRICUBIC_HEXAHEDRON               37

/*parameter structure*/
typedef struct {
  int quite;
  char shape_func_name[32];
  char *filename;
  int cellx;
  int celly;
  double xlength;
  double ylength;
  double zlength;
  int xnum;
  int ynum;
  int znum;
  double dx;
  double dy;
  double dz;
  int flag_pdata;
  int flag_elev;
  int celltype;
} parameters;

/*the elevation function pointer*/
typedef  double (*ef_pointer)(int, int, int, int, int, int, double, double, double);
/*the shape function pointer*/
typedef  void (*sf_pointer)(int, int, int, int, int, int, double, double, double, double*);

extern int get_number_of_cells_per_volume(int celltype);
extern int  get_number_of_points_of_cell (int celltype);
extern void print_ugrid_header (FILE * out);
extern void print_ugrid_cells (FILE * out, parameters *params);
extern void print_ugrid_points (FILE * out, parameters *params);
extern void print_ugrid_point_data (FILE * out, parameters *params);
extern void print_ugrid_cell_data (FILE * out, parameters *params);
extern void compute_node_numbers (parameters *params, int *xnum, int *ynum, int *znum);
extern int  print_single_cell (FILE * out, int cols, int rows, int depths, int *xoffste, int *yoffset, int *zoffset,
			      int celltype);
extern parameters *parse_parameter (int argc, char **argv);
extern void print_help (void);

ef_pointer get_elevfunction(parameters *params);
extern double elevfunc_numbering(int xnum, int ynum, int znum, int i, int j, int k, double dx, double dy, double dz);
extern double elevfunc_standard(int xnum, int ynum, int znum, int i, int j, int k, double dx, double dy, double dz);
extern double elevfunc_simple(int xnum, int ynum, int znum, int i, int j, int k, double dx, double dy, double dz);
extern double elevfunc_cosinus(int xnum, int ynum, int znum, int i, int j, int k, double dx, double dy, double dz);
extern double elevfunc_sinus(int xnum, int ynum, int znum, int i, int j, int k, double dx, double dy, double dz);
extern double elevfunc_parabel(int xnum, int ynum, int znum, int i, int j, int k, double dx, double dy, double dz);

sf_pointer get_shapefunction(parameters *params);
void shapefunc_standard(int xnum, int ynum, int znum, int i, int j, int k, double dx, double dy, double dz, double *x);
void shapefunc_simple(int xnum, int ynum, int znum, int i, int j, int k, double dx, double dy, double dz, double *x);
void shapefunc_cosinus(int xnum, int ynum, int znum, int i, int j, int k, double dx, double dy, double dz, double *x);
void shapefunc_sinus(int xnum, int ynum, int znum, int i, int j, int k, double dx, double dy, double dz, double *x);
void shapefunc_parabel(int xnum, int ynum, int znum, int i, int j, int k, double dx, double dy, double dz, double *x);
void shapefunc_hyperboloid(int xnum, int ynum, int znum, int i, int j, int k, double dx, double dy, double dz, double *x);
void shapefunc_screw(int xnum, int ynum, int znum, int i, int j, int k, double dx, double dy, double dz, double *x);
   
#endif
