/*
 * PROGRAM:
 * vtkcellgen
 * 
 * AUTHOR:
 * Soeren Gebbert
 * soerengebbert <at> googlemail.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *
 * The GNU General Public License is available here:
 * http://www.gnu.org/licenses/gpl.txt
 * 
 */


#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include "globals.h"

/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
parameters *
parse_parameter (int argc, char **argv)
{
  int i, j, stringlength;
  int xnum, ynum, znum;
  char *buff;
  char *token;
  int found = 0;
  parameters *params = calloc(1, sizeof(parameters));

  /*default values */
  params->celltype = DEFAULT_CELL;
  params->cellx = CELLNUMX;
  params->celly = CELLNUMY;
  params->xlength = XLENGTH;
  params->ylength = YLENGTH;
  params->zlength = ZLENGTH;
  params->flag_elev = 0;
  params->flag_pdata = 0;
  params->quite = 0;
  params->filename = NULL;

  /*parse the arguments */
  for (i = 1; i < argc; i++)
    {
      found = 0;
      stringlength = strlen (argv[i]);
      if (strncmp (argv[i], "--", 2) == 0)
	{

	  buff = (char *) calloc (stringlength, sizeof (char));
	  token = (char *) calloc (stringlength, sizeof (char));

	  for (j = 0; j < stringlength; j++)
	    {
	      buff[j] = 0;
	      token[j] = 0;
	    }

	  /*copy string */
	  for (j = 2; j < stringlength; j++)
	    buff[j - 2] = argv[i][j];

	  strcat (buff, "\0");

	  if (strncmp (buff, "cellx=", 6) == 0)
	    {
	      for (j = 6; j < stringlength; j++)
		token[j - 6] = buff[j];

	      sscanf (token, "%i",&(params->cellx));
	      found = 1;
	    }

	  if (strncmp (buff, "celly=", 6) == 0)
	    {
	      for (j = 6; j < stringlength; j++)
		token[j - 6] = buff[j];

	      sscanf (token, "%i", &(params->celly));
	      found = 1;
	    }

	  if (strncmp (buff, "xlength=", 8) == 0)
	    {
	      for (j = 8; j < stringlength; j++)
		token[j - 8] = buff[j];

	      sscanf (token, "%lf", &(params->xlength));
	      found = 1;
	    }

	  if (strncmp (buff, "ylength=", 8) == 0)
	    {
	      for (j = 8; j < stringlength; j++)
		token[j - 8] = buff[j];

	      sscanf (token, "%lf", &(params->ylength));
	      found = 1;
	    }

	  if (strncmp (buff, "zlength=", 8) == 0)
	    {
	      for (j = 8; j < stringlength; j++)
		token[j - 8] = buff[j];

	      sscanf (token, "%lf", &(params->zlength));
	      found = 1;
	    }

	  if (strncmp (buff, "output=", 7) == 0)
	    {
	      for (j = 7; j < stringlength; j++)
		token[j - 7] = buff[j];

	      if (params->filename == NULL)
		{
		  params->filename = (char *) calloc (stringlength, sizeof (char));
		}
	      sscanf (token, "%s", params->filename);
	      found = 1;
	    }

	  if (strncmp (buff, "celltype=", 9) == 0)
	    {
	      for (j = 9; j < stringlength; j++)
		token[j - 9] = buff[j];

	      sscanf (token, "%i", &(params->celltype));
	      found = 1;
	    }

	  if (strncmp (buff, "shapefunc=", 10) == 0)
	    {
	      for (j = 10; j < stringlength; j++)
		token[j - 10] = buff[j];

	      sscanf (token, "%s", params->shape_func_name);
	      found = 1;
	    }

	  if (strncmp (buff, "help", 4) == 0)
	    {
	      print_help ();
	      exit (0);
	    }

          free (buff);
	  free (token);
	}
      else if (strncmp (argv[i], "-", 1) == 0)
	{
	  for (j = 1; j < stringlength; j++)
	    {
	      if (argv[i][j] == 'z')
		{
		  params->flag_elev = 1;
	          found = 1;
		}
	      /*
	      if (argv[i][j] == 'p')
		{
		  params->flag_pdata = 1;
	          found = 1;
		}
	      */
	      if (argv[i][j] == 'q')
		{
		  params->quite = 1;
	          found = 1;
		}
	      if (argv[i][j] == 'h')
		{
		  print_help ();
		  exit (0);
		}
	    }
	}
      else if ((strncmp (argv[i], "help", 4) == 0))
	{
	  print_help ();
	  exit (0);
	}
      /*if the option is unknown, tell the user, print help and exit*/	
      if(found == 0) 
	{
	  fprintf(stderr, "Unknown option: %s\n\n", argv[i]);
	  print_help ();
	  exit (0);
	}
    }

  /*compute grid size*/
  compute_node_numbers (params, &xnum, &ynum, &znum);
  params->xnum = xnum;
  params->ynum = ynum;
  params->znum = znum;

  if(xnum > 1)
    params->dx = params->xlength/((double)xnum-1.0);
  else
    params->dx = params->xlength;

  if(ynum > 1)
    params->dy = params->ylength/((double)ynum-1.0);
  else
    params->dz = params->zlength;

  if(znum > 1)
    params->dz = params->zlength/((double)znum-1.0);
  else
    params->dz = params->zlength;

  /*summarize */
  if (params->quite == 0)
    {
      printf ("Cell type %i\n", params->celltype);
      printf ("Number of cells in x-direction: %i\n", params->cellx);
      printf ("Number of cells in y-direction: %i\n", params->celly);
      printf ("Length in x-direction: %f\n", params->xlength);
      printf ("Length in y-direction: %f\n", params->ylength);
      printf ("Length in z-direction: %f\n", params->zlength);
      printf ("Number of points in x-direction: %i\n", params->xnum);
      printf ("Number of points in y-direction: %i\n", params->ynum);
      printf ("Number of points in z-direction: %i\n", params->znum);
      printf ("Distance between points in x-direction: %f\n", params->dx);
      printf ("Distance between points in y-direction: %f\n", params->dy);
      printf ("Distance between points in z-direction: %f\n", params->dz);
      printf ("Elevation function %s\n", params->shape_func_name);
      printf ("Filename: %s\n", params->filename);
      if (params->flag_pdata == 1)
	printf ("Enable point data indices\n");
    }

  return params;
}

